data{
    int<lower=1> N;
    int SUC[N];
    int TRIAL[N];
    int AGE1[N];
    int AGE2[N];
    int AGE3[N];
    int AGE4[N];
    int AGE5[N];
    int AGE6[N];
    real INMIGRANT[N];
    int MALE[N];
    int COMM_TYPE_F[N];
}
parameters{
    real a;
    real<lower=0> dispersion;
    real fishing;
    real inmigrant;
    real male;
    real age1;
    real age2;
    real age3;
    real age4;
    real age5;
    real age6;
    real age1_female;
    real age1_male;
    real age4_female;
    real age7_male;
    real male_inland;
    real female_inland;
    real inland_inmig;
    real inland_resid;
    real male_inmig;
    real female_inmig;
    real female_resid;
}
model{
    vector[N] p_part;
    female_resid ~ normal( 0 , 10 );
    female_inmig ~ normal( 0 , 10 );
    male_inmig ~ normal( 0 , 10 );
    inland_resid ~ normal( 0 , 10 );
    inland_inmig ~ normal( 0 , 10 );
    female_inland ~ normal( 0 , 10 );
    male_inland ~ normal( 0 , 10 );
    age7_male ~ normal( 0 , 10 );
    age4_female ~ normal( 0 , 10 );
    age1_male ~ normal( 0 , 10 );
    age1_female ~ normal( 0 , 10 );
    age6 ~ normal( 0 , 10 );
    age5 ~ normal( 0 , 10 );
    age4 ~ normal( 0 , 10 );
    age3 ~ normal( 0 , 10 );
    age2 ~ normal( 0 , 10 );
    age1 ~ normal( 0 , 10 );
    male ~ normal( 0 , 10 );
    inmigrant ~ normal( 0 , 10 );
    fishing ~ normal( 0 , 10 );
    dispersion ~ exponential( 1 );
    a ~ normal( 0 , 100 );
    for ( i in 1:N ) {
        p_part[i] = a + fishing * COMM_TYPE_F[i] + inmigrant * INMIGRANT[i] + male * MALE[i] +      age1 * AGE1[i] + age2 * AGE2[i] + age3 * AGE3[i] + age4 * AGE4[i] + age5 *      AGE5[i] + age6 * AGE6[i] + age1_female * AGE1[i] * (1 - MALE[i]) + age1_male *      AGE1[i] * MALE[i] + age4_female * AGE4[i] * (1 - MALE[i]) + age7_male *      (1 - AGE1[i] - AGE2[i] - AGE3[i] - AGE4[i] - AGE5[i] - AGE6[i]) * MALE[i] + male_inland *      MALE[i] * (1 - COMM_TYPE_F[i]) + female_inland * (1 - MALE[i]) * (1 -      COMM_TYPE_F[i]) + inland_inmig * (1 - COMM_TYPE_F[i]) * INMIGRANT[i] +      inland_resid * (1 - COMM_TYPE_F[i]) * (1 - INMIGRANT[i]) + male_inmig *      MALE[i] * INMIGRANT[i] + female_inmig * (1 - MALE[i]) * INMIGRANT[i] +      female_resid * (1 - MALE[i]) * (1 - INMIGRANT[i]);
        p_part[i] = inv_logit(p_part[i]);
    }
    SUC ~ beta_binomial( TRIAL , p_part*dispersion , (1-p_part)*dispersion );
}
generated quantities{
    vector[N] p_part;
    for ( i in 1:N ) {
        p_part[i] = a + fishing * COMM_TYPE_F[i] + inmigrant * INMIGRANT[i] + male * MALE[i] +      age1 * AGE1[i] + age2 * AGE2[i] + age3 * AGE3[i] + age4 * AGE4[i] + age5 *      AGE5[i] + age6 * AGE6[i] + age1_female * AGE1[i] * (1 - MALE[i]) + age1_male *      AGE1[i] * MALE[i] + age4_female * AGE4[i] * (1 - MALE[i]) + age7_male *      (1 - AGE1[i] - AGE2[i] - AGE3[i] - AGE4[i] - AGE5[i] - AGE6[i]) * MALE[i] + male_inland *      MALE[i] * (1 - COMM_TYPE_F[i]) + female_inland * (1 - MALE[i]) * (1 -      COMM_TYPE_F[i]) + inland_inmig * (1 - COMM_TYPE_F[i]) * INMIGRANT[i] +      inland_resid * (1 - COMM_TYPE_F[i]) * (1 - INMIGRANT[i]) + male_inmig *      MALE[i] * INMIGRANT[i] + female_inmig * (1 - MALE[i]) * INMIGRANT[i] +      female_resid * (1 - MALE[i]) * (1 - INMIGRANT[i]);
        p_part[i] = inv_logit(p_part[i]);
    }
}


